#!/bin/env dls-python
import PandATest
from collections import OrderedDict


class DaughterCardException(Exception):
    pass


class Encoder(PandATest.PandATest):

    def __init__(self, *args, **kwargs):
        super(Encoder, self).__init__(*args, **kwargs)
        self.test_type = 'Unplugged'

    def test_code(self):

        #(cmd, (expected result, +- bound of result))
        setup_quad_commands = OrderedDict([
            ('INENC{}.BYPASS=On'.format(self.blocknum),            ('OK', 0)),
            ('OUTENC{}.BYPASS=On'.format(self.blocknum),           ('OK', 0)),
            ('INENC{}.PROTOCOL=Quadrature'.format(self.blocknum),  ('OK', 0)),
            ('OUTENC{}.PROTOCOL=Quadrature'.format(self.blocknum), ('OK', 0))
        ])
        setup_ssi_commands = OrderedDict([
            ('INENC{}.PROTOCOL=SSI'.format(self.blocknum),  ('OK', 0)),
            ('OUTENC{}.PROTOCOL=SSI'.format(self.blocknum), ('OK', 0))
        ])

        #SET FOR QUADRATURE
        for cmd, expected in setup_quad_commands.items():
            response = self.send_cmd(cmd)
            self.test_result.append(
                self.check_response(response, expected[0], cmd, expected[1]))

        self.outputs = ['A', 'B', 'Z']
        self.inputs = self.outputs
        self.ch_connect = {
            self.outputs[0]: -1,
            self.outputs[1]: -1,
            self.outputs[2]: -1,
        }
        default_val = 1
        self.toggle_val('OUTENC', '', 'INENC', '', default_val)

        #SET FOR SSI
        for cmd, expected in setup_ssi_commands.items():
            response = self.send_cmd(cmd)
            self.test_result.append(
                self.check_response(response, expected[0], cmd, expected[1]))

        # #TOGGLE SSI FIELDS
        self.outputs = ['CLK']
        self.inputs = self.outputs
        self.ch_connect = {
            self.outputs[0]: -1,
        }
        self.toggle_val('INENC', '', 'OUTENC', '', default_val)

        self.outputs = ['DATA']
        self.inputs = self.outputs
        self.ch_connect = {
            self.outputs[0]: -1,
        }
        self.toggle_val('OUTENC', '', 'INENC', '', default_val)

    def check_daughter_board(self):
        #CHECK FOR CONTROL ENCODERS
        response = self.send_cmd('INENC{}.DCARD_ID?'.format(self.blocknum))
        if 'Encoder Control' not in response:
            print "Encoder {} daughter board mismatch: {}".format(self.blocknum, response)
            return 1
        else:
            return 0

    def toggle_val(self, out_param, out_field, in_param, in_field, default):
        for outp in self.outputs:
            for val in [1, 0]:
                out_cmd = '{}{}.{}={}'.format(
                    out_param, self.blocknum, outp, self.toggle_cmd[val])
                self.set_output(out_cmd)
                in_cmd = '{}{}.{}?'.format(in_param, self.blocknum, '{}')
                self.toggle_check(
                    in_cmd, outp, self.ch_connect[outp], default, val)
