#!/bin/env dls-python
import PandATest
import Lvds
import Ttl
import Encoder
import os

"""
LVDS and TTL Loop Back (LB) tests combined
"""


class LoopBack(PandATest.PandATest):

    def __init__(self, *args, **kwargs):
        super(LoopBack, self).__init__(*args, **kwargs)
        # LVDS LOOP BACK
        self.lp = Lvds.Lvds(*args, **kwargs)
        self.lp.outputs = [1, 2]
        self.lp.inputs = [1, 2]
        self.lp.ch_connect = {
            self.lp.outputs[0]: self.lp.inputs[0],
            self.lp.outputs[1]: self.lp.inputs[1]
        }
        self.lp.default_val = 0
        self.lp.unplugged = False

        # TTL LOOP BACK
        self.tp = Ttl.Ttl(*args, **kwargs)
        self.tp.outputs = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        self.tp.inputs = [1, 2, 3, 4, 5, 6]
        self.tp.ch_connect = {
            self.tp.outputs[0]: self.tp.inputs[0],
            self.tp.outputs[1]: self.tp.inputs[1],
            self.tp.outputs[2]: self.tp.inputs[2],
            self.tp.outputs[3]: self.tp.inputs[3],
            self.tp.outputs[4]: self.tp.inputs[4],
            self.tp.outputs[5]: self.tp.inputs[5],
            self.tp.outputs[6]: self.tp.inputs[0],
            self.tp.outputs[7]: self.tp.inputs[1],
            self.tp.outputs[8]: self.tp.inputs[2],
            self.tp.outputs[9]: self.tp.inputs[3]
        }
        self.tp.default_val = 0

        #ENCODER LOOP BACK
        self.encx = []
        for enc_num in range(1, 5):
            self.encx.append(Encoder.Encoder(*args, **kwargs))
            self.encx[enc_num - 1].blocknum = enc_num

    def check_setup(self):
        print "Checking Encoder Daughter Cards..."
        enc_ok = True
        for enc_num in range(4):
            if self.encx[enc_num].check_daughter_board():
                enc_ok = False
        if not enc_ok:
            raise Encoder.DaughterCardException(
                "Power down PandA and connect control daughter board")
        else:
            print "Daughter Cards OK.."

    def test_code(self):
        self.run_loopback_test(self.lp)
        self.run_loopback_test(self.tp)
        for enc_num in range(4):
            self.run_loopback_test(self.encx[enc_num])


    def run_loopback_test(self, test_cls):
        self.logger.info('---- {} {} {} {}'.format(test_cls.__class__.__name__,
                                                   test_cls.blocknum,
                                                   'LOOP BACK',
                                                   ''.join(['-']*28)
                                                   ))
        for attempt in [1, 2]:
            self.logger.info('---- ATTEMPT {} {}'.format(attempt, ''.join(['-']*28)))
            test_cls.test_code()
            if test_cls.test_result.count(0) != 0 and attempt == 1:
                test_cls.test_result = []
                os.system(
                    "./load-state {} base.state".format(self.hostname))
                self.check_connectors(test_cls.__class__.__name__, test_cls.blocknum)
            elif test_cls.test_result.count(0) == 0:
                print 'TEST PASSED! ({} {})'.format(test_cls.__class__.__name__,
                                                    test_cls.blocknum)
                break

    def get_summary(self):
        tests = [self.lp, self.tp]
        for enc_num in range(4):
            tests.append(self.encx[enc_num])
        summary = []
        for test in tests:
            summary.append("{:<12}{:<12}{:<10} ERRORS: {}".format(
                                        test.__class__.__name__,
                                        test.unplugged_string[test.unplugged],
                                        test.blocknum,
                                        test.test_result.count(0)))
        return summary

    def get_user_input(self):
        inputy = '.'
        while inputy not in ['']:
            print "Loopback connectors connected, OK ? [return]"
            inputy = raw_input()

    def check_connectors(self, test, block_no):
        inputy = '.'
        while inputy not in ['']:
            print "TEST FAILED! ({} {}), Check connectors connected [return]".format(test, block_no)
            inputy = raw_input()






