#!/bin/env dls-python
import PandATest
import Lvds
import Ttl
import EncoderUnplugged as Encoder
import os

"""
LVDS and TTL Unplugged tests
"""


class Unplugged(PandATest.PandATest):

    def __init__(self, *args, **kwargs):
        super(Unplugged, self).__init__(*args, **kwargs)
        # LVDS UNPLUGGED
        self.lu = Lvds.Lvds(*args, **kwargs)
        self.lu .inputs = [1, 2]
        self.lu .outputs = [1, 2]
        self.lu .ch_connect = {
            self.lu .outputs[0]: -1,
            self.lu .outputs[1]: -1
        }
        self.default_val = 1
        self.lu.unplugged = True

        #TTL UNPLUGGED
        self.tu = Ttl.Ttl(*args, **kwargs)
        self.tu.outputs = [1,2,3,4,5,6,7,8,9,10]
        self.tu.inputs = [1,2,3,4,5,6]
        self.tu.ch_connect = {
            self.tu.outputs[0]: -1,
            self.tu.outputs[1]: -1,
            self.tu.outputs[2]: -1,
            self.tu.outputs[3]: -1,
            self.tu.outputs[4]: -1,
            self.tu.outputs[5]: -1,
            self.tu.outputs[6]: -1,
            self.tu.outputs[7]: -1,
            self.tu.outputs[8]: -1,
            self.tu.outputs[9]: -1,
        }
        self.tu.default_val = 0
        self.tu.unplugged = True

        #ENCODER UNPLUGGED
        self.encx = []
        for enc_num in range(1, 5):
            self.encx.append(Encoder.Encoder(*args, **kwargs))
            self.encx[enc_num - 1].blocknum = enc_num
            self.encx[enc_num - 1].unplugged = True

    def check_setup(self):
        print "Checking Encoder Daughter Cards..."
        enc_ok = True
        for enc_num in range(4):
            if self.encx[enc_num].check_daughter_board():
                enc_ok = False
        if not enc_ok:
            raise Encoder.DaughterCardException(
                "Power down PandA and connect control daughter board")
        else:
            print "Daughter Cards OK.."

    def test_code(self):
        self.run_unplugged_test(self.lu)
        self.run_unplugged_test(self.tu)
        for enc_num in range(4):
            self.run_unplugged_test(self.encx[enc_num])

    def run_unplugged_test(self, test_cls):
        self.logger.info('---- {} {} {} {}'.format(test_cls.__class__.__name__,
                                                   test_cls.blocknum,
                                                   'UNPLUGGED',
                                                   ''.join(['-']*28)
                                                   ))
        for attempt in [1, 2]:
            self.logger.info('---- ATTEMPT {} {}'.format(
                attempt, ''.join(['-']*28)))
            test_cls.test_code()
            if test_cls.test_result.count(0) != 0 and attempt == 1:
                test_cls.test_result = []
                os.system(
                    "./load-state {} base.state".format(self.hostname))
                self.check_connectors(
                    test_cls.__class__.__name__, test_cls.blocknum)
            elif test_cls.test_result.count(0) == 0:
                print 'TEST PASSED! ({} {})'.format(test_cls.__class__.__name__,
                                                    test_cls.blocknum)
                break

    def get_summary(self):
        tests = [self.lu, self.tu]
        for enc_num in range(4):
            tests.append(self.encx[enc_num])
        summary = []
        for test in tests:
            summary.append("{:<12}{:<12}{:<10} ERRORS: {}".format(
                                        test.__class__.__name__,
                                        test.unplugged_string[test.unplugged],
                                        test.blocknum,
                                        test.test_result.count(0)))
        return summary

    def get_user_input(self):
        inputy = '.'
        while inputy not in ['']:
            print "Loopback connectors NOT connected, OK ? [return]"
            inputy = raw_input()

    def check_connectors(self, test, block_no):
        inputy = '.'
        while inputy not in ['']:
            print "TEST FAILED! ({} {})," \
                  " Check connectors connected [return]".format(test, block_no)
            inputy = raw_input()
